//======================================================================
//-----------------------------------------------------------------------
/**
 * @file		WXRichTextParams.h
 * @brief		b`GfBbgRg[gp̍\̋@\gt@C
 *
 * @author		t.sirayanagi
 * @version		1.0
 *
 * @par			copyright
 * Copyright (C) 2009-2010 Takazumi Shirayanagi\n
 * The new BSD License is applied to this software.
 * see iris_LICENSE.txt
*/
//-----------------------------------------------------------------------
//======================================================================
#ifndef _IRIS_WXRichTextParams_H_
#define _IRIS_WXRichTextParams_H_

//======================================================================
// include
#include "iris_object.h"
#include <richedit.h>

namespace iris {
namespace wx
{

//======================================================================
// declare
template<typename _TN>class ICharFormat;
template<typename _TN>class ICharFormat2;
template<typename _TN>class IParaFormat;
template<typename _TN>class IParaFormat2;

//======================================================================
// typedef
typedef class ICharFormat<CHARFORMATA>		CCharFormatA;	//!< CHARFORMATANX
typedef class ICharFormat<CHARFORMATW>		CCharFormatW;	//!< CHARFORMATWNX
typedef class ICharFormat2<CHARFORMAT2A>	CCharFormat2A;	//!< CHARFORMAT2ANX
typedef class ICharFormat2<CHARFORMAT2W>	CCharFormat2W;	//!< CHARFORMAT2WNX

typedef class IParaFormat<PARAFORMAT>		CParaFormat;	//!< PARAFORMATNX
typedef class IParaFormat2<PARAFORMAT2>		CParaFormat2;	//!< PARAFORMAT2NX

#ifdef UNICODE
typedef CCharFormatW	CCharFormat;	//!< CHARFORMATNX
typedef CCharFormat2W	CCharFormat2;	//!< CHARFORMATNX
#else
typedef CCharFormatA	CCharFormat;	//!< CHARFORMATNX
typedef CCharFormat2A	CCharFormat2;	//!< CHARFORMATNX
#endif

//======================================================================
// function

//======================================================================
// class
//! CHARFORMATev[gNX
template<typename _TN>
class ICharFormat : public IIrisObject
{
	typedef _TN	Format;
protected:
	Format		m_Fmt;	//! CHARFORMAT
public:
	// RXgN^
	ICharFormat(void);
	ICharFormat(const Format& rFormat);

	// fXgN^
	~ICharFormat(void);

public:
	/// tH[}bg̎擾
	Format&	GetFormat(void)			{ return m_Fmt; }
	Format&	operator () (void)		{ return GetFormat(); }
	operator Format& (void)			{ return GetFormat(); }

protected:
	// Lɂ
	void		EnableEffects(DWORD dwCFM, DWORD dwCFE);
	// ɂ
	void		DisableEffects(DWORD dwCFM, DWORD dwCFE);
	// Lǂ
	bool		IsEffects(DWORD dwCFM, DWORD dwCFE);
public:
	/// ݒɂ
	void		EnableBold(void)			{ EnableEffects(CFM_BOLD, CFE_BOLD); }
	/// ݒ
	void		DisableBold(void)			{ DisableEffects(CFM_BOLD, CFE_BOLD); }
	/// ݒ肩ǂ
	bool		IsBold(void)				{ return IsEffects(CFM_BOLD, CFE_BOLD); }
	/// ݒ
	void		FlagBold(bool enable)		{ enable ? EnableBold() : DisableBold(); }

	/// Α̐ݒɂ
	void		EnableItalic(void)			{ EnableEffects(CFM_ITALIC, CFE_ITALIC); }
	/// Α̐ݒ
	void		DisableItalic(void)			{ DisableEffects(CFM_ITALIC, CFE_ITALIC); }
	/// Α̐ݒ肩ǂ
	bool		IsItalic(void)				{ return IsEffects(CFM_ITALIC, CFE_ITALIC); }
	/// Α̐ݒ
	void		FlagItalic(bool enable)		{ enable ? EnableItalic() : DisableItalic(); }

	/// ݒɂ
	void		EnableUnderline(void)		{ EnableEffects(CFM_UNDERLINE, CFE_UNDERLINE); }
	/// ݒ
	void		DisableUnderline(void)		{ DisableEffects(CFM_UNDERLINE, CFE_UNDERLINE); }
	/// ݒ肩ǂ
	bool		IsUnderline(void)			{ return IsEffects(CFM_UNDERLINE, CFE_UNDERLINE); }
	/// ݒ
	void		FlagUnderline(bool enable)	{ enable ? EnableUnderline() : DisableUnderline(); }

	/// łݒɂ
	void		EnableStrikeOut(void)		{ EnableEffects(CFM_STRIKEOUT, CFE_STRIKEOUT); }
	/// łݒ
	void		DisableStrikeOut(void)		{ DisableEffects(CFM_STRIKEOUT, CFE_STRIKEOUT); }
	/// łݒ肩ǂ
	bool		IsStrikeOut(void)			{ return IsEffects(CFM_STRIKEOUT, CFE_STRIKEOUT); }
	/// łݒ
	void		FlagStrikeOut(bool enable)	{ enable ? EnableStrikeOut() : DisableStrikeOut(); }

	/// یݒɂ
	void		EnableProtected(void)		{ EnableEffects(CFM_PROTECTED, CFE_PROTECTED); }
	/// یݒ
	void		DisableProtected(void)		{ DisableEffects(CFM_PROTECTED, CFE_PROTECTED); }
	/// یݒ肩ǂ
	bool		IsProtected(void)			{ return IsEffects(CFM_PROTECTED, CFE_PROTECTED); }
	/// یݒ
	void		FlagProtected(bool enable)	{ enable ? EnableProtected() : DisableProtected(); }

	/// Nݒɂ
	void		EnableLink(void)			{ EnableEffects(CFM_LINK, CFE_LINK); }
	/// Nݒ
	void		DisableLink(void)			{ DisableEffects(CFM_LINK, CFE_LINK); }
	/// Nݒ肩ǂ
	bool		IsLink(void)				{ return IsEffects(CFM_LINK, CFE_LINK); }
	/// Nݒ
	void		FlagLink(bool enable)		{ enable ? EnableLink() : DisableLink(); }

public:
	// ̍ݒ(twipP)
	void		SetCharHeightTwip(LONG lHeight);
	// ̍擾(twipP)
	LONG		GetCharHeightTwip(void);
	// ̍ݒ(pixelP)
	void		SetCharHeightPixel(HWND hWnd, LONG lHeight);
	// ̍擾(pixelP)
	LONG		GetCharHeightPixel(HWND hWnd);
	// ̍ݒ(pointP)
	void		SetCharHeightPoint(LONG lHeight);
	// ̍擾(pointP)
	LONG		GetCharHeightPoint(void);
	// ̍ݒLɂ
	void		EnableCharHeight(void);
	// ̍ݒ𖳌ɂ
	void		DisableCharHeight(void);
	// ̍ݒ肪Lǂ
	bool		IsCharHeight(void);
	/// ̍ݒ
	void		FlagCharHeight(bool enable)		{ enable ? EnableCharHeight() : DisableCharHeight(); }

	// LN^Zbgݒ
	void		SetCharSet(BYTE byCharSet);
	// LN^Zbg擾
	BYTE		GetCharSet(void);
	// LN^ZbgLɂ
	void		EnableCharSet(void);
	// LN^Zbg𖳌ɂ
	void		DisableCharSet(void);
	// LN^ZbgLǂ
	bool		IsCharSet(void);
	/// LN^Zbg
	void		FlagCharSet(bool enable)		{ enable ? EnableCharSet() : DisableCharSet(); }

	// x[XC̃ItZbgݒ
	void		SetOffset(LONG lOffset);
	// x[XC̃ItZbg擾
	LONG		GetOffset(void);
	// x[XC̃ItZbgݒLɂ
	void		EnableOffset(void);
	// x[XC̃ItZbgݒ𖳌ɂ
	void		DisableOffset(void);
	// x[XC̃ItZbgݒ肪Lǂ
	bool		IsOffset(void);
	/// x[XC̃ItZbgݒ
	void		FlagOffset(bool enable)		{ enable ? EnableOffset() : DisableOffset(); }

	// Fݒ
	void		SetTextColor(COLORREF color);
	// F擾
	COLORREF	GetTextColor(void);
	// F̐ݒLɂ
	void		EnableTextColor(void);
	// F̐ݒ𖳌ɂ
	void		DisableTextColor(void);
	// F̐ݒLǂ
	bool		IsTextColor(void);
	/// F̐ݒ
	void		FlagTextColor(bool enable)		{ enable ? EnableTextColor() : DisableTextColor(); }
	// FݒVXeJ[ɂ
	void		EnableAutoTextColor(void);
	// FݒVXeJ[ɂ̂
	void		DisableAutoTextColor(void);
	// Fݒ肪VXeJ[ݒ肩ǂ
	bool		IsAutoTextColor(void);
	/// VXeJ[ݒ
	void		FlagAutoTextColor(bool enable)	{ enable ? EnableAutoTextColor() : DisableAutoTextColor(); }

	// tHg̐ݒ
	void		SetFontFace(LPVOID lpszFace);
	// tHg̐ݒLɂ
	void		EnableFontFace(void);
	// tHg̐ݒ𖳌ɂ
	void		DisableFontFace(void);
	// tHg̐ݒLǂ
	bool		IsFontFace(void);
	/// tHg̐ݒ
	void		FlagFontFace(bool enable)		{ enable ? EnableFontFace() : DisableFontFace(); }

public:
	// tHgt@~ƃsb`w
	void		SetPitchAndFamily(BYTE byPitchAndFamily);
	// tHgt@~ƃsb`擾
	BYTE		GetPitchAndFamily(void);
};

//! CHARFORMAT2ev[gNX
template<typename _TN>
class ICharFormat2 : public ICharFormat<_TN>
{
	typedef ICharFormat<_TN>	_Myt;
public:
	/// ݒɂiRg[ɂ\ɉeȂj
	void		EnableSmallCaps(void)		{ EnableEffects(CFM_SMALLCAPS, CFE_SMALLCAPS); }
	/// ݒiRg[ɂ\ɉeȂj
	void		DisableSmallCaps(void)		{ DisableEffects(CFM_SMALLCAPS, CFE_SMALLCAPS); }
	/// ݒ肩ǂiRg[ɂ\ɉeȂj
	bool		IsSmallCaps(void)			{ return IsEffects(CFM_SMALLCAPS, CFE_SMALLCAPS); }
	/// ݒiRg[ɂ\ɉeȂj
	void		FlagSmallCaps(bool enable)	{ enable ? EnableSmallCaps() : DisableSmallCaps(); }

	/// 啶ݒɂiRg[ɂ\ɉeȂj
	void		EnableAllCaps(void)			{ EnableEffects(CFM_ALLCAPS, CFE_ALLCAPS); }
	/// 啶ݒiRg[ɂ\ɉeȂj
	void		DisableAllCaps(void)		{ DisableEffects(CFM_ALLCAPS, CFE_ALLCAPS); }
	/// 啶ݒ肩ǂiRg[ɂ\ɉeȂj
	bool		IsAllCaps(void)				{ return IsEffects(CFM_ALLCAPS, CFE_ALLCAPS); }
	/// 啶ݒiRg[ɂ\ɉeȂj
	void		FlagAllCaps(bool enable)	{ enable ? EnableAllCaps() : DisableAllCaps(); }

	/// \ݒɂi3.0ȍ~j
	void		EnableHidden(void)			{ EnableEffects(CFM_HIDDEN, CFE_HIDDEN); }
	/// \ݒi3.0ȍ~j
	void		DisableHidden(void)			{ DisableEffects(CFM_HIDDEN, CFE_HIDDEN); }
	/// \ݒ肩ǂi3.0ȍ~j
	bool		IsHidden(void)				{ return IsEffects(CFM_HIDDEN, CFE_HIDDEN); }
	/// \ݒ
	void		FlagHidden(bool enable)		{ enable ? EnableHidden() : DisableHidden(); }

	/// ֊s`ꂽݒɂiRg[ɂ\ɉeȂj
	void		EnableOutline(void)			{ EnableEffects(CFM_OUTLINE, CFE_OUTLINE); }
	/// ֊s`ꂽݒiRg[ɂ\ɉeȂj
	void		DisableOutline(void)		{ DisableEffects(CFM_OUTLINE, CFE_OUTLINE); }
	/// ֊s`ꂽݒ肩ǂiRg[ɂ\ɉeȂj
	bool		IsOutline(void)				{ return IsEffects(CFM_OUTLINE, CFE_OUTLINE); }
	/// ֊s`ꂽݒiRg[ɂ\ɉeȂj
	void		FlagOutline(bool enable)	{ enable ? EnableOutline() : DisableOutline(); }

	/// AeݒɂiRg[ɂ\ɉeȂj
	void		EnableShadow(void)			{ EnableEffects(CFM_SHADOW, CFE_SHADOW); }
	/// AeݒiRg[ɂ\ɉeȂj
	void		DisableShadow(void)			{ DisableEffects(CFM_SHADOW, CFE_SHADOW); }
	/// Aeݒ肩ǂiRg[ɂ\ɉeȂj
	bool		IsShadow(void)				{ return IsEffects(CFM_SHADOW, CFE_SHADOW); }
	/// AeݒiRg[ɂ\ɉeȂj
	void		FlagShadow(bool enable)		{ enable ? EnableShadow() : DisableShadow(); }

	/// オݒɂiRg[ɂ\ɉeȂj
	void		EnableEmboss(void)			{ EnableEffects(CFM_EMBOSS, CFE_EMBOSS); }
	/// オݒiRg[ɂ\ɉeȂj
	void		DisableEmboss(void)			{ DisableEffects(CFM_EMBOSS, CFE_EMBOSS); }
	/// オݒ肩ǂiRg[ɂ\ɉeȂj
	bool		IsEmboss(void)				{ return IsEffects(CFM_EMBOSS, CFE_EMBOSS); }
	/// オݒiRg[ɂ\ɉeȂj
	void		FlagEmboss(bool enable)		{ enable ? EnableEmboss() : DisableEmboss(); }

	/// 󂳂ꂽݒɂiRg[ɂ\ɉeȂj
	void		EnableImprint(void)			{ EnableEffects(CFM_IMPRINT, CFE_IMPRINT); }
	/// 󂳂ꂽݒiRg[ɂ\ɉeȂj
	void		DisableImprint(void)		{ DisableEffects(CFM_IMPRINT, CFE_IMPRINT); }
	/// 󂳂ꂽݒ肩ǂiRg[ɂ\ɉeȂj
	bool		IsImprint(void)				{ return IsEffects(CFM_IMPRINT, CFE_IMPRINT); }
	/// 󂳂ꂽݒiRg[ɂ\ɉeȂj
	void		FlagImprint(bool enable)	{ enable ? EnableImprint() : DisableImprint(); }

	/// e\ݒɂ
	void		EnableDisabled(void)		{ EnableEffects(CFM_DISABLED, CFE_DISABLED); }
	/// e\ݒ
	void		DisableDisabled(void)		{ DisableEffects(CFM_DISABLED, CFE_DISABLED); }
	/// e\ݒ肩ǂ
	bool		IsDisabled(void)			{ return IsEffects(CFM_DISABLED, CFE_DISABLED); }
	/// e\ݒ
	void		FlagDisabled(bool enable)	{ enable ? EnableDisabled() : DisableDisabled(); }

	/// }[N\ݒɂ
	void		EnableRevised(void)			{ EnableEffects(CFM_REVISED, CFE_REVISED); }
	/// }[N\ݒ
	void		DisableRevised(void)		{ DisableEffects(CFM_REVISED, CFE_REVISED); }
	/// }[N\ݒ肩ǂ
	bool		IsRevised(void)				{ return IsEffects(CFM_REVISED, CFE_REVISED); }
	/// }[N\ݒ
	void		FlagRevised(bool enable)	{ enable ? EnableRevised() : DisableRevised(); }

	/// tݒɂ
	void		EnableSubScript(void)		{ EnableEffects(CFM_SUBSCRIPT, CFE_SUBSCRIPT); }
	/// tݒ
	void		DisableSubScript(void)		{ DisableEffects(CFM_SUBSCRIPT, CFE_SUBSCRIPT); }
	/// tݒ肩ǂ
	bool		IsSubScript(void)			{ return IsEffects(CFM_SUBSCRIPT, CFE_SUBSCRIPT); }
	/// tݒ
	void		FlagSubScript(bool enable)	{ enable ? EnableSubScript() : DisableSubScript(); }

	/// tݒɂ
	void		EnableSuperScript(void)		{ EnableEffects(CFM_SUPERSCRIPT, CFE_SUPERSCRIPT); }
	/// tݒ
	void		DisableSuperScript(void)	{ DisableEffects(CFM_SUPERSCRIPT, CFE_SUPERSCRIPT); }
	/// tݒ肩ǂ
	bool		IsSuperScript(void)			{ return IsEffects(CFM_SUPERSCRIPT, CFE_SUPERSCRIPT); }
	/// tݒ
	void		FlagSuperScript(bool enable){ enable ? EnableSuperScript() : DisableSuperScript(); }

public:
	// wiFݒ
	void		SetBackColor(COLORREF color);
	// wiF擾
	COLORREF	GetBackColor(void);
	// wiF̐ݒLɂ
	void		EnableBackColor(void);
	// wiF̐ݒ𖳌ɂ
	void		DisableBackColor(void);
	// wiF̐ݒLǂ
	bool		IsBackColor(void);
	/// wiF̐ݒ
	void		FlagBackColor(bool enable)	{ enable ? EnableBackColor() : DisableBackColor(); }
	// wiFݒVXeJ[ɂ
	void		EnableAutoBackColor(void);
	// wiFݒVXeJ[ɂ̂
	void		DisableAutoBackColor(void);
	// wiFݒ肪VXeJ[ݒ肩ǂ
	bool		IsAutoBackColor(void);
	/// VXeJ[ݒ
	void		FlagAutoBackColor(bool enable)	{ enable ? EnableAutoBackColor() : DisableAutoBackColor(); }

public:
	// tHg̑ݒ
	void		SetWeight(WORD wWeight);
	// tHg̑擾
	WORD		GetWeight(void);
	// tHg̑Lɂ
	void		EnableWeight(void);
	// tHg̑𖳌ɂ
	void		DisableWeight(void);
	// tHg̑Lǂ
	bool		IsWeight(void);
	/// tHg̑
	void		FlagWeight(bool enable)				{ enable ? EnableWeight() : DisableWeight(); }

	// ̃^Cvݒ
	void		SetUnderlineType(BYTE byUnderlineType);
	// ̃^Cv擾
	BYTE		GetUnderlineType(void);
	// ̃^CvݒLɂ
	void		EnableUnderlineType(void);
	// ̃^Cvݒ𖳌ɂ
	void		DisableUnderlineType(void);
	// ̃^Cvݒ肪Lǂ
	bool		IsUnderlineType(void);
	/// ̃^Cvݒ
	void		FlagUnderlineType(bool enable)		{ enable ? EnableUnderlineType() : DisableUnderlineType(); }

	// ԊuݒiRg[ɂ\ɉeȂj
	void		SetSpacing(SHORT sSpacing);
	// Ԋu擾iRg[ɂ\ɉeȂj
	SHORT		GetSpacing(void);
	// ԊuݒLɂiRg[ɂ\ɉeȂj
	void		EnableSpacing(void);
	// Ԋuݒ𖳌ɂiRg[ɂ\ɉeȂj
	void		DisableSpacing(void);
	// Ԋuݒ肪LǂiRg[ɂ\ɉeȂj
	bool		IsSpacing(void);
	/// ԊuݒiRg[ɂ\ɉeȂj
	void		FlagSpacing(bool enable)			{ enable ? EnableSpacing() : DisableSpacing(); }

	// [PIDݒiRg[ɂ\ɉeȂj
	void		SetLCID(DWORD lcid);
	// [PID擾iRg[ɂ\ɉeȂj
	DWORD		GetLCID(void);
	// [PIDݒLɂiRg[ɂ\ɉeȂj
	void		EnableLCID(void);
	// [PIDݒ𖳌ɂiRg[ɂ\ɉeȂj
	void		DisableLCID(void);
	// [PIDݒ肪LǂiRg[ɂ\ɉeȂj
	bool		IsLCID(void);
	/// [PIDݒiRg[ɂ\ɉeȂj
	void		FlagLCID(bool enable)				{ enable ? EnableLCID() : DisableLCID(); }

	// X^CݒiRg[ɂ\ɉeȂj
	void		SetCharStyle(SHORT sStyle);
	// X^C擾iRg[ɂ\ɉeȂj
	SHORT		GetCharStyle(void);
	// X^CݒLɂiRg[ɂ\ɉeȂj
	void		EnableCharStyle(void);
	// X^Cݒ𖳌ɂiRg[ɂ\ɉeȂj
	void		DisableCharStyle(void);
	// X^Cݒ肪LǂiRg[ɂ\ɉeȂj
	bool		IsCharStyle(void);
	///X^CݒiRg[ɂ\ɉeȂj
	void		FlagCharStyle(bool enable)				{ enable ? EnableCharStyle() : DisableCharStyle(); }

	// J[jOݒiRg[ɂ\ɉeȂj
	void		SetKerning(WORD sStyle);
	// J[jO擾iRg[ɂ\ɉeȂj
	WORD		GetKerning(void);
	// J[jOݒLɂiRg[ɂ\ɉeȂj
	void		EnableKerning(void);
	// J[jOݒ𖳌ɂiRg[ɂ\ɉeȂj
	void		DisableKerning(void);
	// J[jOݒ肪LǂiRg[ɂ\ɉeȂj
	bool		IsKerning(void);
	///J[jOݒiRg[ɂ\ɉeȂj
	void		FlagKerning(bool enable)				{ enable ? EnableKerning() : DisableKerning(); }
};

//! PARAFORMATNX
template<typename _TN>
class IParaFormat : public IIrisObject
{
	typedef _TN	Format;
protected:
	Format		m_Fmt;	//! PARAFORMAT
public:
	// RXgN^
	IParaFormat(void);
	IParaFormat(const Format& rFormat);
	// fXgN^
	~IParaFormat(void);

public:
	/// tH[}bg̎擾
	Format&	GetFormat(void)			{ return m_Fmt; }
	Format&	operator () (void)		{ return GetFormat(); }
	operator Format& (void)			{ return GetFormat(); }

public:
	// w肳ꂽi̍ŏ̍s̃Cfgݒ
	void		SetStartIndent(LONG lIndent);
	// w肳ꂽi̍ŏ̍s̃Cfg擾
	LONG		GetStartIndent(void);
	// w肳ꂽi̍ŏ̍s̃Cfgݒi΁jLɂ
	void		EnableStartIndent(void);
	// w肳ꂽi̍ŏ̍s̃Cfgݒi΁j𖳌ɂ
	void		DisableStartIndent(void);
	// w肳ꂽi̍ŏ̍s̃Cfgݒi΁jLǂ
	bool		IsStartIndent(void);
	// w肳ꂽi̍ŏ̍s̃Cfgݒi΁jLɂ
	void		EnableOffsetIndent(void);
	// w肳ꂽi̍ŏ̍s̃Cfgݒi΁j𖳌ɂ
	void		DisableOffsetIndent(void);
	// w肳ꂽi̍ŏ̍s̃Cfgݒi΁jLǂ
	bool		IsOffsetIndent(void);

	// w肳ꂽỉẼCfgݒ
	void		SetRightIndent(LONG lIndent);
	// w肳ꂽỉẼCfg擾
	LONG		GetRightIndent(void);
	// w肳ꂽỉẼCfgݒLɂ
	void		EnableRightIndent(void);
	// w肳ꂽỉẼCfgݒ𖳌ɂ
	void		DisableRightIndent(void);
	// w肳ꂽỉẼCfgݒ肪Lǂ
	bool		IsRightIndent(void);

	// 2sڈȍ~̍s̃Cfgݒ
	void		SetOffset(LONG lIndent);
	// 2sڈȍ~̍s̃Cfg擾
	LONG		GetOffset(void);
	// 2sڈȍ~̍s̃CfgݒLɂ
	void		EnableOffset(void);
	// 2sڈȍ~̍s̃Cfgݒ𖳌ɂ
	void		DisableOffset(void);
	// 2sڈȍ~̍s̃Cfgݒ肪Lǂ
	bool		IsOffset(void);

	// i̔zuݒ
	void		SetAlignment(WORD wAlignment);
	// i̔zu擾
	WORD		GetAlignment(void);
	// i̔zuݒLɂ
	void		EnableAlignment(void);
	// i̔zuݒ𖳌ɂ
	void		DisableAlignment(void);
	// i̔zuݒ肪Lǂ
	bool		IsAlignment(void);

	// ^uXgbv̐ݒ
	void		SetTabCount(SHORT sTabCount);
	// ^uXgbv̐擾
	SHORT		GetTabCount(void);
	// ^uXgbv̈ʒuݒ
	void		SetTabPos(int nIndex, LONG lPos);
	// ^uXgbv̈ʒu擾
	LONG		GetTabPos(int nIndex);
	// ^uXgbvݒLɂ
	void		EnableTabStops(void);
	// ^uXgbvݒ𖳌ɂ
	void		DisableTabStops(void);
	// ^uXgbvݒ肪Lǂ
	bool		IsTabStops(void);

	// ioOIvVݒ
	void		SetNumbering(WORD wNumbering);
	// ioOIvV擾
	WORD		GetNumbering(void);
	// ioOIvVݒLɂ
	void		EnableNumbering(void);
	// ioOIvVݒ𖳌ɂ
	void		DisableNumbering(void);
	// ioOIvVݒ肪Lǂ
	bool		IsNumbering(void);

	// ǐʂݒiRg[ɂ\ɉeȂj
	void		SetRtlPara(WORD sStyle);
	// ǐʂ擾iRg[ɂ\ɉeȂj
	WORD		GetRtlPara(void);
	// ǐʐݒLɂiRg[ɂ\ɉeȂj
	void		EnableRtlPara(void);
	// ǐʐݒ𖳌ɂiRg[ɂ\ɉeȂj
	void		DisableRtlPara(void);
	// ǐʊuݒ肪LǂiRg[ɂ\ɉeȂj
	bool		IsRtlPara(void);
};

//! PARAFORMAT2NX
template<typename _TN>
class IParaFormat2 : public IParaFormat<_TN>
{
};

}	// end of namespace wx
}	// end of namespace iris

#endif
